import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import { addSeconds } from 'date-fns';

import { AUTH_REDUCER_NAME, AuthAttributes, AuthRootState, AuthTokens, ModifyAuthAttributePayload } from './types';

const initialState: AuthAttributes = {
	authenticated: undefined,
	accessToken: undefined,
	refreshToken: undefined,
	idToken: undefined,
	accessExpiresIn: undefined,
	refreshExpiresIn: undefined,
	accessExpiresTime: undefined,
	refreshExpiresTime: undefined,
	sessionState: undefined,
};

const authSlice = createSlice({
	name: AUTH_REDUCER_NAME,
	initialState,
	reducers: {
		setAuthTokens: (_, action: PayloadAction<AuthTokens>) => {
			const { accessExpiresIn, refreshExpiresIn } = action.payload;
			return {
				...action.payload,
				...(accessExpiresIn ? { accessExpiresTime: addSeconds(new Date(), accessExpiresIn).toISOString() } : {}),
				...(refreshExpiresIn ? { refreshExpiresTime: addSeconds(new Date(), refreshExpiresIn).toISOString() } : {}),
				authenticated: true,
			};
		},
		setAuthAttribute: (state, action: PayloadAction<AuthAttributes>) => {
			return {
				...state,
				...action.payload,
			};
		},
		updateAuthAttribute: (state, action: PayloadAction<ModifyAuthAttributePayload>) => {
			state[action.payload.key] = action.payload.value;
		},
		deleteAuthAttributes: () => {
			return {};
		},
	},
});

export const { setAuthTokens, deleteAuthAttributes } = authSlice.actions;

export const setAuthAttribute = authSlice.actions.setAuthAttribute as <T>(
	attr: AuthAttributes<T>
) => PayloadAction<AuthAttributes<T>>;

export const updateAuthAttribute = authSlice.actions.updateAuthAttribute as <T>(
	attrs: ModifyAuthAttributePayload<T>
) => PayloadAction<ModifyAuthAttributePayload<T>>;

export const getAuthAttribute = <T>(state: AuthRootState, name: keyof AuthAttributes<T>) =>
	state.procyon.auth[name as any];

export default authSlice.reducer;
